//+------------------------------------------------------------------+
//|                                                          Pin.mq4 |
//|                                         Copyright 2015, mrak297. |
//|                            https://www.mql5.com/ru/users/mrak297 |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, mrak297."
#property link      "https://www.mql5.com/ru/users/mrak297"
#property version   "15.08"
#property strict
#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 clrBlue
#property indicator_color2 clrRed

//--- input parameters
extern ENUM_TIMEFRAMES TimeFrame        = PERIOD_CURRENT;    // Time frame to use
input int    Depth    = 40;   //Depth of pinbars in percents
input int    MaxRange = 100;  //Maximum range
input int    MinRange = 20;   //Minimum range
input bool   Extremum = true; //Only extremum
extern bool  AlertsMessage = true; 
extern bool  AlertsSound   = false;
extern bool  AlertsEmail   = false;
extern bool  AlertsMobile  = false;

int   SignalBar = 1;
datetime TimeBar;
double Pp;
int bar;

double buf[];
double Up[];
double Dn[];
string fileName;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
  bar=0;
   if (Point() == 0.0001 || Point() == 0.00001) Pp = 0.0001;
   if (Point() == 0.01 || Point() == 0.001) Pp = 0.01;
   
   SetIndexBuffer(0, Up);
   SetIndexBuffer(1, Dn);
   
   SetIndexStyle(0, DRAW_ARROW,STYLE_SOLID, 1);
   SetIndexStyle(1, DRAW_ARROW,STYLE_SOLID, 1);
   
   SetIndexArrow(0,233);
   SetIndexArrow(1,234);
   
   SetIndexLabel(0, "UP");
   SetIndexLabel(1, "DOWN");
      TimeFrame = MathMax(TimeFrame,_Period);
      fileName  = WindowExpertName();
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   if (TimeFrame!=_Period)
   {
      for (int i=Bars-1; i>=0; i--)
      {
         int y =              iBarShift(NULL,TimeFrame,Time[i]);
         int x = (i<Bars-1) ? iBarShift(NULL,TimeFrame,Time[i+1]): y;
            Dn[i]=EMPTY_VALUE;
            Up[i]=EMPTY_VALUE;
            if (x!=y)
            {
               Up[i] = iCustom(NULL,TimeFrame,fileName,PERIOD_CURRENT,Depth,MaxRange,MinRange,Extremum,AlertsMessage,AlertsSound,AlertsEmail,AlertsMobile,0,y);
               Dn[i] = iCustom(NULL,TimeFrame,fileName,PERIOD_CURRENT,Depth,MaxRange,MinRange,Extremum,AlertsMessage,AlertsSound,AlertsEmail,AlertsMobile,1,y);
            }
      }
      return(0);
   }            
   
//if(bar!=Bars)
//{


   string name = " ";
   
  
   for (int i = 1; i < Bars-1; i++)
    {
      Dn[i]=EMPTY_VALUE;
      Up[i]=EMPTY_VALUE;
      if (PinUp(time[i], open[i], high[i], low[i], close[i], Depth))   Dn[i] = high[i];
      if (PinDown(time[i], open[i], high[i], low[i], close[i], Depth)) Up[i] = low[i];
    }
    
    
string namez = " ";
namez = "PinUp"+TimeToString(Time[1], TIME_DATE|TIME_MINUTES); 
string nameq = " ";
nameq = "PinDown"+TimeToString(Time[1], TIME_DATE|TIME_MINUTES); 
    bar=Bars;
//    }
//-------------------------------------------------------------------+   
 if(AlertsMessage || AlertsSound || AlertsEmail || AlertsMobile)
  { 
   string message1 = (WindowExpertName()+" - "+Symbol()+"  "+PeriodString()+" - PinUp");
   string message2 = (WindowExpertName()+" - "+Symbol()+"  "+PeriodString()+" - PinDown");
       
    if(TimeBar!=Time[0] && Up[SignalBar]!=0 && Up[SignalBar]!=EMPTY_VALUE)
     { 
        if (AlertsMessage) Alert(message1);
        if (AlertsSound)   PlaySound("alert2.wav");
        if (AlertsEmail)   SendMail(Symbol()+" - "+WindowExpertName()+" - ",message1);
        if (AlertsMobile)  SendNotification(message1);
        TimeBar=Time[0];
     }
    if(TimeBar!=Time[0] && Dn[SignalBar]!=0 && Dn[SignalBar]!=EMPTY_VALUE)
     { 
        if (AlertsMessage) Alert(message2);
        if (AlertsSound)   PlaySound("alert2.wav");
        if (AlertsEmail)   SendMail(Symbol()+" - "+WindowExpertName()+" - ",message2);
        if (AlertsMobile)  SendNotification(message2);
        TimeBar=Time[0];
    }
  }
//-------------------------------------------------------------------+
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
 {
  ObjectsDeleteAll(0, -1, OBJ_ARROW);
 }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool PinUp(datetime time, double open, double high, double low, double close, int depth)
 {
  int  range = (int)MathRound((high - low)/Pp);
  int  zone  = (int)MathRound(range * depth * 0.01);                      //////////////////////////////////////
  int  shift = iBarShift(_Symbol, _Period, time, true);                 //                                  //
  double dayHidh = iHigh(_Symbol, _Period, shift);                      //       |               --|        //
  double level = NormalizeDouble(low + zone * Pp, _Digits);               //       |                 |        //
  bool check = false;                                                     //       |                 |        //
                                                                          //       |                 |        //
  if (range > MinRange && range < MaxRange)                               //       |                 |        //
   {                                                                      //       |                 |- range //
    if ((Extremum && high == dayHidh) || !Extremum)                       //       |                 |        //
     check = true;                                                        //      ---     --|- level |        //
    if (open <= level && close <= level && check)                         //      | |       |        |        //
     {                                                                    //      | |       |-- zone |        //
      return(true);                                                       //      ---       |        |        //
     }                                                                    //       |      --|      --|        //
   }                                                                      //                                  //
                                                                          //////////////////////////////////////
  return(false);
 }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool PinDown(datetime time, double open, double high, double low, double close, int depth)
 {
  int  range = (int)MathRound((high - low)/Pp);
  int  zone  = (int)MathRound(range * depth * 0.01);                      //////////////////////////////////////
  int  shift = iBarShift(_Symbol, _Period, time, true);                 //                                  //
  double dayLow = iLow(_Symbol, _Period, shift);                        //       |      --|      --|        //
  double level = NormalizeDouble(high - zone * Pp, _Digits);              //      ---       |        |        //
  bool check = false;                                                     //      | |       |-- zone |        //
                                                                          //      | |       |        |        //
  if (range > MinRange && range < MaxRange)                               //      ---     --|- level |        //
   {                                                                      //       |                 |        //
    if ((Extremum && low == dayLow) || !Extremum)                         //       |                 |- range //
     check = true;                                                        //       |                 |        //
    if (open >= level && close >= level && check)                         //       |                 |        //
     {                                                                    //       |                 |        //
      return(true);                                                       //       |                 |        //
     }                                                                    //       |               --|        //
   }                                                                      //                                  //
                                                                          //////////////////////////////////////
  return(false);
}
//-------------------------------------------------------------------+ 
string PeriodString()
{
    switch (_Period) 
     {
        case PERIOD_M1:  return("M1");
        case PERIOD_M5:  return("M5");
        case PERIOD_M15: return("M15");
        case PERIOD_M30: return("M30");
        case PERIOD_H1:  return("H1");
        case PERIOD_H4:  return("H4");
        case PERIOD_D1:  return("D1");
        case PERIOD_W1:  return("W1");
        case PERIOD_MN1: return("MN1");
     }    
   return("M" + string(_Period));
}
//-------------------------------------------------------------------+